/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include <cmath>
#include <bitset>

#include "drivers/stereo_elec/StereoElecDef.h"

#include "DAQParamController.h"
#include "controllers/common/family/Families.h"

using namespace std;

namespace stereo_daq {

const string DAQParamController::TYPE = "daqparam_controller";

/*
 * Constructor
 */
DAQParamController::DAQParamController(const string& name) :
acquisition::TimeAcquisitionController(name), Select(this),Set(this) {

	setFamily(family::ACQUISITION,family::SETTING);

	// Init property
	daqmode.init(this, NOSAVE, "daqmode");
	trigmode.init(this, NOSAVE, "trigmode");
	reqcoinc.init(this, NOSAVE, "reqcoinc");
	nsample.init(this, NOSAVE, "nsample");
	fileformat.init(this, NOSAVE, "fileformat");
	filtercoef.init(this, NOSAVE, "filtercoef");
	ledused.init(this, NOSAVE, "ledused");
	nfeboard.init(this, NOSAVE, "nfeboard");
	ntot.init(this, NOSAVE, "ntot");
	ntail.init(this, NOSAVE, "ntail");
	pretrig.init(this, NOSAVE, "pretrig");
	cfddelay.init(this, NOSAVE, "cfddelay");
	cfdfraction.init(this, NOSAVE, "cfdfraction");
	precfd.init(this, NOSAVE, "precfd");
	trigchannel.init(this, NOSAVE, "trigchannel");
	trigsum03.init(this, NOSAVE, "trigsum03");
	trigsum47.init(this, NOSAVE, "trigsum47");
	trigsum07.init(this, NOSAVE, "trigsum07");
	trigquantity.init(this, NOSAVE, "trigquantity");
	slotindex.init(this, NOSAVE, "slotindex");
	sumthres.init(this, NOSAVE, "sumthres");
	nsum.init(this, NOSAVE, "nsum");
	t1accepted.init(this, NOSAVE, "t1accepted");
	t1timereset.init(this, NOSAVE, "t1timereset");

	ntotchannel.init(this, NOSAVE, "ntotchannel");
	islot.init(this, NOSAVE, "islot");
	ichannel.init(this, NOSAVE, "ichannel");
	namechannel.init(this, NOSAVE, "namechannel");
	gain20.init(this, NOSAVE, "gain20");
	trigthres.init(this, NOSAVE, "trigthres");
	cfdthres.init(this, NOSAVE, "cfdthres");
	isselected.init(this, NOSAVE, "isselected");

	selectchoice.init(this,NOSAVE,"selectchoice");
	selectslot.init(this,NOSAVE,"selectslot");
	setparam.init(this, NOSAVE, "setparam");
	setvalue.init(this, NOSAVE, "setvalue");

	trigthresall.init(this, NOSAVE, "trigthresall");

	m_HVDriver.init(this, "hvdriver");

	m_StartActivated = false;

	registerFunction(TYPE);

	registerRefresher(nfeboard,&DAQParamController::refreshNFEBoardProperty,this);

}

DAQParamController::DAQParamController(const DAQParamController& controller) :
		acquisition::TimeAcquisitionController(controller), Select(this),Set(this) {

	daqmode.copy(this, controller.daqmode);
	trigmode.copy(this, controller.trigmode);
	reqcoinc.copy(this, controller.reqcoinc);
	nsample.copy(this, controller.nsample);
	fileformat.copy(this, controller.fileformat);
	filtercoef.copy(this, controller.filtercoef);
	ledused.copy(this, controller.ledused);
	nfeboard.copy(this, controller.nfeboard);
	ntot.copy(this, controller.ntot);
	ntail.copy(this, controller.ntail);
	pretrig.copy(this, controller.pretrig);
	cfddelay.copy(this, controller.cfddelay);
	cfdfraction.copy(this, controller.cfdfraction);
	precfd.copy(this, controller.precfd);
	trigchannel.copy(this, controller.trigchannel);
	trigsum03.copy(this, controller.trigsum03);
	trigsum47.copy(this, controller.trigsum47);
	trigsum07.copy(this, controller.trigsum07);
	trigquantity.copy(this, controller.trigquantity);
	slotindex.copy(this, controller.slotindex);
	sumthres.copy(this, controller.sumthres);
	nsum.copy(this, controller.nsum);
	t1accepted.copy(this, controller.t1accepted);
	t1timereset.copy(this,controller.t1timereset);


	ntotchannel.copy(this, controller.ntotchannel);
	islot.copy(this, controller.islot);
	ichannel.copy(this, controller.ichannel);
	namechannel.copy(this, controller.namechannel);
	gain20.copy(this, controller.gain20);
	trigthres.copy(this, controller.trigthres);
	cfdthres.copy(this, controller.cfdthres);
	isselected.copy(this, controller.isselected);

	selectchoice.copy(this, controller.selectchoice);
	selectslot.copy(this, controller.selectslot);
	setparam.copy(this, controller.setparam);
	setvalue.copy(this, controller.setvalue);

	trigthresall.copy(this, controller.trigthresall);

	m_HVDriver.copy(controller.m_HVDriver);

	m_StartActivated = false;
}

/*
 * Destructor
 */
DAQParamController::~DAQParamController() {
	// Empty
}

void DAQParamController::postConfiguration() {

	TimeAcquisitionController::postConfiguration();

	m_ELECDriver.copy(m_Drivers[0]);

	registerRefresher(daqmode, &DAQParamController::refreshDAQModeProperty, this);
	registerRefresher(trigmode, &DAQParamController::refreshTrigModeProperty, this);
	registerRefresher(reqcoinc, &DAQParamController::refreshReqCoincProperty, this);
	registerRefresher(nsample, &DAQParamController::refreshNSampleProperty, this);
	registerRefresher(fileformat, &DAQParamController::refreshFileFormatProperty, this);
	registerRefresher(filtercoef, &DAQParamController::refreshFilterCoefProperty, this);
	registerRefresher(ledused, &DAQParamController::refreshLedUsedProperty, this);
	registerRefresher(ntot, &DAQParamController::refreshNTotProperty, this);
	registerRefresher(ntail, &DAQParamController::refreshNTailProperty, this);
	registerRefresher(pretrig, &DAQParamController::refreshPretrigProperty, this);
	registerRefresher(trigquantity, &DAQParamController::refreshTrigQuantityProperty, this);
	registerRefresher(trigchannel, &DAQParamController::refreshTrigChannelProperty, this);
	registerRefresher(trigsum03, &DAQParamController::refreshTrigSum03Property, this);
	registerRefresher(trigsum47, &DAQParamController::refreshTrigSum47Property, this);
	registerRefresher(trigsum07, &DAQParamController::refreshTrigSum07Property, this);
	registerRefresher(cfddelay, &DAQParamController::refreshCFDDelayProperty, this);
	registerRefresher(cfdfraction, &DAQParamController::refreshCFDFractionProperty, this);
	registerRefresher(gain20, &DAQParamController::refreshGain20Property, this);
	registerRefresher(trigthres, &DAQParamController::refreshTrigThresProperty, this);
	registerRefresher(cfdthres, &DAQParamController::refreshCFDThresProperty, this);
	registerRefresher(sumthres, &DAQParamController::refreshSumThresProperty, this);
	registerRefresher(precfd, &DAQParamController::refreshPreCFDProperty, this);
	registerRefresher(nsum, &DAQParamController::refreshNSumProperty, this);
	registerRefresher(t1accepted, &DAQParamController::refreshT1AcceptedProperty, this);
	registerRefresher(t1timereset, &DAQParamController::refreshT1TimeResetProperty, this);
	registerRefresher(namechannel, &DAQParamController::refreshNameChannelProperty, this);
	registerRefresher(trigthresall, &DAQParamController::refreshTrigThresAllProperty, this);

	registerUpdater(m_ELECDriver->triggeripaddress, &DAQParamController::updateTriggerIPAddress, this);
	registerUpdater(m_ELECDriver->nfeboard, &DAQParamController::updateNFEBoard, this);
	registerUpdater(m_ELECDriver->boardipaddress, &DAQParamController::updateBoardIPAddress, this);
	registerUpdater(m_ELECDriver->namechannel0, &DAQParamController::updateNameChannel0, this);
	registerUpdater(m_ELECDriver->namechannel1, &DAQParamController::updateNameChannel1, this);
	registerUpdater(m_ELECDriver->namechannel2, &DAQParamController::updateNameChannel2, this);
	registerUpdater(m_ELECDriver->namechannel3, &DAQParamController::updateNameChannel3, this);
	registerUpdater(m_ELECDriver->namechannel4, &DAQParamController::updateNameChannel4, this);
	registerUpdater(m_ELECDriver->namechannel5, &DAQParamController::updateNameChannel5, this);
	registerUpdater(m_ELECDriver->namechannel6, &DAQParamController::updateNameChannel6, this);
	registerUpdater(m_ELECDriver->namechannel7, &DAQParamController::updateNameChannel7, this);
	registerUpdater(m_ELECDriver->startActivated, &DAQParamController::updateStartActivated, this);

	nfeboard = m_ELECDriver->nfeboard();
	m_ELECDriver->DAQParams.triggerIPAdd = m_ELECDriver->triggeripaddress();
	m_ELECDriver->DAQParams.isNatR5=false;
	daqmode = 1;
	trigmode = 0;
	reqcoinc = 0;
	nsample = 60;
	fileformat="TEXT";
	filtercoef=130928;
	ledused=0;
	m_ELECDriver->DAQParams.ledUsed=false;
	trigthresall=0;
	setparam="";
	setvalue="0";

}

void DAQParamController::refreshNFEBoardProperty(int32 value) throw (CannotSetValue) {
	log(Level::s_Info) << "Number of front end changed from " << nfeboard() << " to " << value << endlog;

	// Resize properties and vector
	ntot.resize(value);
	ntail.resize(value);
	pretrig.resize(value);
	cfddelay.resize(value);
	cfdfraction.resize(value);
	precfd.resize(value);
	trigchannel.resize(value);
	trigsum03.resize(value);
	trigsum47.resize(value);
	trigsum07.resize(value);
	trigquantity.resize(value);
	slotindex.resize(value);
	sumthres.resize(value);
	nsum.resize(value);
	t1accepted.resize(value);
	t1timereset.resize(value);

	ntotchannel.set(value * stereo_elec::NBFECHANNEL);
	cout << "ntotchannel " << ntotchannel() << endl;
	islot.resize(ntotchannel());
	ichannel.resize(ntotchannel());
	namechannel.resize(ntotchannel());
	gain20.resize(ntotchannel());
	trigthres.resize(ntotchannel());
	cfdthres.resize(ntotchannel());
	isselected.resize(ntotchannel());

	m_ELECDriver->DAQParams.elecMapping_v.resize(ntotchannel());
	m_ELECDriver->DAQParams.slotHV_v.resize(ntotchannel());
	m_ELECDriver->DAQParams.channelHV_v.resize(ntotchannel());
	//cout << "test "  << value << " " << sizeof(STFEParams) << endl;

	m_ELECDriver->DAQParams.FEParams_v.resize(value);
	for (int i = 0; i < value; i++) {
		m_ELECDriver->DAQParams.FEParams_v.at(i).gain_v.resize(stereo_elec::NBFECHANNEL);
		m_ELECDriver->DAQParams.FEParams_v.at(i).threshold_v.resize(stereo_elec::NBFECHANNEL);
		m_ELECDriver->DAQParams.FEParams_v.at(i).CFDThreshold_v.resize(stereo_elec::NBFECHANNEL);
	}

	// initialisation slotindex property, IP address
	int32 index = 0;

	cout << "nslot " << m_ELECDriver->nslot() << endl;
	for (int j = 0; j < m_ELECDriver->nslot(); j++) {
		cout << "j " << j << "index " << index<< " " << m_ELECDriver->boardtype(j) << endl;
		m_ELECDriver->DAQParams.isFEBoard_v.at(j) = false;
		if (m_ELECDriver->boardtype(j) == stereo_elec::FEBOARD) {
			m_ELECDriver->DAQParams.isFEBoard_v.at(j) = true;
			m_ELECDriver->DAQParams.FEParams_v.at(index).FE8IPAdd=m_ELECDriver->boardipaddress(j);
			m_ELECDriver->DAQParams.FEParams_v.at(index).slot_num = j+1;
			slotindex.set(index, j);
			cout << "IP " << m_ELECDriver->DAQParams.FEParams_v.at(index).FE8IPAdd << endl;
			index++;

		}
		else if (m_ELECDriver->boardtype(j) == stereo_elec::LEDBOARD) {
			m_ELECDriver->DAQParams.ledBoardInserted =true;
			m_ELECDriver->DAQParams.ledIPAdd=m_ELECDriver->boardipaddress(j);
		}

		cout << "IsFeboard " << j << " " << m_ELECDriver->DAQParams.isFEBoard_v.at(j) << endl;
	}
//	m_ELECDriver->DAQParams.fe_trigger_mask=1023;
//	m_ELECDriver->DAQParams.fe_reset_mask=0;
//	cout << "fe_trigger_mask 2 "<<  m_ELECDriver->DAQParams.fe_trigger_mask <<  endl;

	// initialisation properties with default values, except for namechannel and ip address
	index = 0;
	for (int i = 0; i < value; i++) {
		ntot.set(i, 50);
		ntail.set(i, 40);
		pretrig.set(i, 10);
		cfddelay.set(i, 2);
		cfdfraction.set(i, 13107);
		precfd.set(i, 2);
		trigchannel.set(i, true);
		trigsum03.set(i, false);
		trigsum47.set(i, false);
		trigsum07.set(i, false);
		trigquantity.set(i, TrigAmp);
		sumthres.set(i, 1000);
		nsum.set(i, 50);
		t1accepted.set(i,true);
		t1timereset.set(i,false);

		for (int k = 0; k < stereo_elec::NBFECHANNEL; k++) {
			islot.set(index, slotindex(i));
			ichannel.set(index, k);
			switch (k) {
			case 0:
				namechannel.set(index, m_ELECDriver->namechannel0(slotindex(i)));
				break;
			case 1:
				namechannel.set(index, m_ELECDriver->namechannel1(slotindex(i)));
				break;
			case 2:
				namechannel.set(index, m_ELECDriver->namechannel2(slotindex(i)));
				break;
			case 3:
				namechannel.set(index, m_ELECDriver->namechannel3(slotindex(i)));
				break;
			case 4:
				namechannel.set(index, m_ELECDriver->namechannel4(slotindex(i)));
				break;
			case 5:
				namechannel.set(index, m_ELECDriver->namechannel5(slotindex(i)));
				break;
			case 6:
				namechannel.set(index, m_ELECDriver->namechannel6(slotindex(i)));
				break;
			case 7:
				namechannel.set(index, m_ELECDriver->namechannel7(slotindex(i)));
				break;
			}
			isselected.set(index, false);
			gain20.set(index, false);
			trigthres.set(index, 100);
			cfdthres.set(index, 100);

			index++;
		}
	}

}
void DAQParamController::refreshDAQModeProperty(int32 value) throw (CannotSetValue) {
	m_ELECDriver->DAQParams.DAQMode = (short unsigned) value;
}
void DAQParamController::refreshTrigModeProperty(int32 value) throw (CannotSetValue) {
	m_ELECDriver->DAQParams.trigMode = (short unsigned) value;
}
void DAQParamController::refreshReqCoincProperty(int32 value) throw (CannotSetValue) {
	m_ELECDriver->DAQParams.reqCoinc = (short unsigned) value;
}
void DAQParamController::refreshNSampleProperty(int32 value) throw (CannotSetValue) {
	m_ELECDriver->DAQParams.nSamples = (short unsigned) value;
}
void DAQParamController::refreshFileFormatProperty(const std::string& value) throw (CannotSetValue) {
	m_ELECDriver->DAQParams.file_format = value;
}
void DAQParamController::refreshFilterCoefProperty(int value) throw (CannotSetValue) {
	m_ELECDriver->DAQParams.afrac = value;
}
void DAQParamController::refreshLedUsedProperty(int value) throw (CannotSetValue) {
	m_ELECDriver->DAQParams.ledUsed = value;
}
void DAQParamController::refreshNTotProperty(int32 index, int32 value) throw (CannotSetValue) {
	m_ELECDriver->DAQParams.FEParams_v.at(index).nTot = (short unsigned) value;
}

void DAQParamController::refreshNTailProperty(int32 index, int32 value) throw (CannotSetValue) {
	m_ELECDriver->DAQParams.FEParams_v.at(index).nTail = (short unsigned) value;
}

void DAQParamController::refreshPretrigProperty(int32 index, int32 value) throw (CannotSetValue) {
	m_ELECDriver->DAQParams.FEParams_v.at(index).preTrig = (short unsigned) value;
}

void DAQParamController::refreshTrigQuantityProperty(int32 index, const std::string& value) throw (CannotSetValue) {
	if (value == TrigSum)
		m_ELECDriver->DAQParams.FEParams_v.at(index).isTriggerOnQ = true;
	else
		m_ELECDriver->DAQParams.FEParams_v.at(index).isTriggerOnQ = false;
}

void DAQParamController::refreshTrigChannelProperty(int32 index, bool value) throw (CannotSetValue) {
	if (value)
		m_ELECDriver->DAQParams.FEParams_v.at(index).trigMask |= 0x00FF; // mise à 1 des 8 premiers bits
	else
		m_ELECDriver->DAQParams.FEParams_v.at(index).trigMask &= 0xFF00; // mise à 0 des 8 premiers bits
	cout << "trigMask " << (bitset<16> ) m_ELECDriver->DAQParams.FEParams_v.at(index).trigMask << endl;
}

void DAQParamController::refreshTrigSum03Property(int32 index, bool value) throw (CannotSetValue) {
	if (value)
		m_ELECDriver->DAQParams.FEParams_v.at(index).trigMask |= (short unsigned) (1u << 8); // mise à 1 du bit 8
	else
		m_ELECDriver->DAQParams.FEParams_v.at(index).trigMask &= (short unsigned) ~(1u << 8); // mise à 0 du bit 8
	cout << "trigMask " << (bitset<16> ) m_ELECDriver->DAQParams.FEParams_v.at(index).trigMask << endl;
}

void DAQParamController::refreshTrigSum47Property(int32 index, bool value) throw (CannotSetValue) {
	if (value)
		m_ELECDriver->DAQParams.FEParams_v.at(index).trigMask |= (short unsigned) (1u << 9); // mise à 1 du bit 9
	else
		m_ELECDriver->DAQParams.FEParams_v.at(index).trigMask &= (short unsigned) ~(1u << 9); // mise à 0 du bit 9
	cout << "trigMask " << (bitset<16> ) m_ELECDriver->DAQParams.FEParams_v.at(index).trigMask << endl;
}

void DAQParamController::refreshTrigSum07Property(int32 index, bool value) throw (CannotSetValue) {
	if (value)
		m_ELECDriver->DAQParams.FEParams_v.at(index).trigMask |= (short unsigned) (1u << 10); // mise à 1 du bit 10
	else
		m_ELECDriver->DAQParams.FEParams_v.at(index).trigMask &= (short unsigned) ~(1u << 10); // mise à 0 du bit 10
	cout << "trigMask " << (bitset<16> ) m_ELECDriver->DAQParams.FEParams_v.at(index).trigMask << endl;
}

void DAQParamController::refreshCFDDelayProperty(int32 index, int32 value) throw (CannotSetValue) {
	m_ELECDriver->DAQParams.FEParams_v.at(index).CFDDelay = (short unsigned) value;
}

void DAQParamController::refreshCFDFractionProperty(int32 index, int32 value) throw (CannotSetValue) {
	m_ELECDriver->DAQParams.FEParams_v.at(index).CFDFrac = (short unsigned) value;
}

void DAQParamController::refreshPreCFDProperty(int32 index, int32 value) throw (CannotSetValue) {
	m_ELECDriver->DAQParams.FEParams_v.at(index).preCFD = (short unsigned) value;
}

void DAQParamController::refreshSumThresProperty(int32 index, int32 value) throw (CannotSetValue) {
	m_ELECDriver->DAQParams.FEParams_v.at(index).sumThreshold = value;
}
void DAQParamController::refreshNSumProperty(int32 index, int32 value) throw (CannotSetValue) {
	m_ELECDriver->DAQParams.FEParams_v.at(index).nSum = (short unsigned) value;
}

void DAQParamController::refreshT1AcceptedProperty(int32 index, bool value) throw (CannotSetValue) {
	//if(value) m_ELECDriver->DAQParams.fe_trigger_mask|=  (1 << slotindex(index));
	//else m_ELECDriver->DAQParams.fe_trigger_mask &= ~(1<< slotindex(index));
	m_ELECDriver->DAQParams.fe_trigger_mask=1023;
	cout << "fe_trigger_mask "<<  m_ELECDriver->DAQParams.fe_trigger_mask << " value=" << value << endl;
}
void DAQParamController::refreshT1TimeResetProperty(int32 index, bool value) throw (CannotSetValue) {
	//if(value) m_ELECDriver->DAQParams.fe_reset_mask|=  (1 << slotindex(index));
	//else m_ELECDriver->DAQParams.fe_reset_mask &=  ~(1<< slotindex(index));
	m_ELECDriver->DAQParams.fe_reset_mask=0;
	cout << "fe_reset_mask "<<  m_ELECDriver->DAQParams.fe_reset_mask << " value=" << value << endl;
}


void DAQParamController::refreshGain20Property(int32 index, bool value) throw (CannotSetValue) {
	//cout << "index " << index << " slot " << index/stereo_elec::NBFECHANNEL << " channel " << ichannel(index) << " value " << value << endl;
	m_ELECDriver->DAQParams.FEParams_v.at(index / stereo_elec::NBFECHANNEL).gain_v[ichannel(index)] = value;
}

void DAQParamController::refreshTrigThresProperty(int32 index, int32 value) throw (CannotSetValue) {
	m_ELECDriver->DAQParams.FEParams_v.at(index / stereo_elec::NBFECHANNEL).threshold_v[ichannel(index)] =
			(short unsigned) value;
}

void DAQParamController::refreshCFDThresProperty(int32 index, int32 value) throw (CannotSetValue) {
	m_ELECDriver->DAQParams.FEParams_v.at(index / stereo_elec::NBFECHANNEL).CFDThreshold_v[ichannel(index)] =
			(short unsigned) value;
}
void DAQParamController::refreshNameChannelProperty(int32 index, const std::string& value) throw (CannotSetValue) {
	m_ELECDriver->DAQParams.elecMapping_v.at(index) = value;

	m_ELECDriver->DAQParams.slotHV_v.at(index) = -1;
	m_ELECDriver->DAQParams.channelHV_v.at(index) = -1;

	int i = 0;
	if (m_HVDriver->ntotchannel() > 0) {
		while (m_HVDriver->namechannel(i) != value && i < m_HVDriver->ntotchannel() - 1) {
			i++;
		}
		if (i < m_HVDriver->ntotchannel()) {
			m_ELECDriver->DAQParams.slotHV_v.at(index) = m_HVDriver->islot(i);
			m_ELECDriver->DAQParams.channelHV_v.at(index) = m_HVDriver->ichannel(i);
		}
	}
}
void DAQParamController::refreshTrigThresAllProperty(int32 value) throw (CannotSetValue) {
	if(value>0){
		for(int i=0;i<nfeboard();i++){
			for(int j=0;j< stereo_elec::NBFECHANNEL; j++){
				m_ELECDriver->DAQParams.FEParams_v.at(i).threshold_v[j] = (short unsigned) value;
			}
		}
		cout << "Trigger threshold for all channels have been set to " << value << endl;
	}
}

void DAQParamController::updateTriggerIPAddress() {
	m_ELECDriver->DAQParams.triggerIPAdd = m_ELECDriver->triggeripaddress();
}

void DAQParamController::updateNFEBoard() {
	nfeboard = m_ELECDriver->nfeboard();
}


void DAQParamController::updateBoardIPAddress(int32 index) {
	if (m_ELECDriver->boardtype(index) == stereo_elec::FEBOARD) {
		m_ELECDriver->DAQParams.FEParams_v.at(m_ELECDriver->feindex.at(index)).FE8IPAdd=m_ELECDriver->boardipaddress(index);
	}
	if (m_ELECDriver->boardtype(index) == stereo_elec::LEDBOARD) {
		m_ELECDriver->DAQParams.ledIPAdd=m_ELECDriver->boardipaddress(index);
	}
}

void DAQParamController::updateNameChannel0(int32 index) {
	if (m_ELECDriver->boardtype(index) == stereo_elec::FEBOARD) {
		namechannel.set(m_ELECDriver->feindex.at(index) * stereo_elec::NBFECHANNEL + 0, m_ELECDriver->namechannel0(index));
	}
}

void DAQParamController::updateNameChannel1(int32 index) {
	if (m_ELECDriver->boardtype(index) == stereo_elec::FEBOARD) {
		namechannel.set(m_ELECDriver->feindex.at(index) * stereo_elec::NBFECHANNEL + 1, m_ELECDriver->namechannel1(index));
	}
}
void DAQParamController::updateNameChannel2(int32 index) {
	if (m_ELECDriver->boardtype(index) == stereo_elec::FEBOARD) {
		namechannel.set(m_ELECDriver->feindex.at(index) * stereo_elec::NBFECHANNEL + 2, m_ELECDriver->namechannel2(index));
	}
}
void DAQParamController::updateNameChannel3(int32 index) {
	if (m_ELECDriver->boardtype(index) == stereo_elec::FEBOARD) {
		namechannel.set(m_ELECDriver->feindex.at(index) * stereo_elec::NBFECHANNEL + 3, m_ELECDriver->namechannel3(index));
	}
}
void DAQParamController::updateNameChannel4(int32 index) {
	if (m_ELECDriver->boardtype(index) == stereo_elec::FEBOARD) {
		namechannel.set(m_ELECDriver->feindex.at(index) * stereo_elec::NBFECHANNEL + 4, m_ELECDriver->namechannel4(index));
	}
}
void DAQParamController::updateNameChannel5(int32 index) {
	if (m_ELECDriver->boardtype(index) == stereo_elec::FEBOARD) {
		namechannel.set(m_ELECDriver->feindex.at(index) * stereo_elec::NBFECHANNEL + 5, m_ELECDriver->namechannel5(index));
	}
}
void DAQParamController::updateNameChannel6(int32 index) {
	if (m_ELECDriver->boardtype(index) == stereo_elec::FEBOARD) {
		namechannel.set(m_ELECDriver->feindex.at(index) * stereo_elec::NBFECHANNEL + 6, m_ELECDriver->namechannel6(index));
	}
}
void DAQParamController::updateNameChannel7(int32 index) {
	if (m_ELECDriver->boardtype(index) == stereo_elec::FEBOARD) {
		namechannel.set(m_ELECDriver->feindex.at(index) * stereo_elec::NBFECHANNEL + 7, m_ELECDriver->namechannel7(index));
	}
}
void DAQParamController::updateStartActivated() {
	m_HVDriver->locked=m_ELECDriver->startActivated();
}

/*
 * select channels
 */
void DAQParamController::select() {
	if (selectchoice() == "all") {
		for (int i = 0; i < ntotchannel(); i++) {
			isselected.set(i, true);
		}
	} else if (selectchoice() == "none") {
		for (int i = 0; i < ntotchannel(); i++) {
			isselected.set(i, false);
		}
	} else if (selectchoice() == "slot") {
		for (int i = 0; i < ntotchannel(); i++) {
			if (islot(i) == selectslot())
				isselected.set(i, true);
			//else isselected.set(i,false);
		}
	}

}

void DAQParamController::set() {
	for (int i = 0; i < ntotchannel(); i++) {
		if (isselected(i) == true) {
			if (setparam() == "Gain20") {
				if (setvalue() == "true")
					gain20.set(i, true);
				else
					gain20.set(i, false);
			} else if (setparam() == "TrigThres")
				trigthres.set(i, strtol(setvalue().c_str(), NULL, 10));
			else if (setparam() == "CFDThres")
				cfdthres.set(i, strtol(setvalue().c_str(), NULL, 10));
		}
	}
}

/*
 * resize
 */
void DAQParamController::resize() throw (CannotOpenFile, SlicesFileError, TooManySlices, SliceTooBig) {
	m_Detector->setDataSize(128, 1);
}

/*
 * writeParams
 */
void DAQParamController::writeParams() {
	commandProgression = 0;
	commandProgression = 100;
}

/*
 * synchronizeRead
 */
void DAQParamController::synchronizeRead() {
	commandProgression = 0;
	commandProgression = 100;
}

}

