/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "RealStereoElecDriver.h"
#include "StereoElecDriver.h"
#include "StereoElecDef.h"

#include <iostream>
#include <boost/thread.hpp>
#include <boost/bind.hpp>


using namespace std;

namespace stereo_elec {


const int32 RealStereoElecDriver::READ_COMMAND = 1;
const int32 RealStereoElecDriver::START_COMMAND = 2;
const int32 RealStereoElecDriver::STOP_COMMAND = 3;
const int32 RealStereoElecDriver::SHUTDOWN_COMMAND = 4;
const int32 RealStereoElecDriver::READINFO_COMMAND = 5;
const int32 RealStereoElecDriver::CONFIG_COMMAND = 6;

/*
 * Constructor
 */
RealStereoElecDriver::RealStereoElecDriver(StereoElecDriver* owner) :
		StereoElecState(owner) {


    m_commandQueue = new common::Queue<int32>(1024);
    m_serverThread = new boost::thread(boost::bind(&RealStereoElecDriver::qtLoop, this));

}

/*
 * Destructor
 */
RealStereoElecDriver::~RealStereoElecDriver() {


	m_commandQueue->put(new int32(SHUTDOWN_COMMAND));
	m_serverThread->join();
	delete m_serverThread;
	delete m_commandQueue;

}

void RealStereoElecDriver::qtLoop() {

	int argc = 0;
	char **argv = 0;
	QCoreApplication qtApplication(argc, argv);

	TstereoDaqLib* daq=new TstereoDaqLib();

	while(true){
        int32 * command = m_commandQueue->get();
        if (*command == SHUTDOWN_COMMAND) {
        	delete daq;
        	break;
        } else if (*command == CONFIG_COMMAND) {
        	daq->createTrigEnMask(owner()->DAQParams);
        	cout << "createTrigEnMask ok" << endl;

        	daq->InitAMCProto(owner()->nfeboard());
        	daq->DoConfig(owner()->DAQParams);
        	cout << "DoConfig ok" << endl;

        	owner()->triggermacaddress=owner()->DAQParams.triggerMACAdd;
        	for(int i=0;i<owner()->nfeboard();i++){
        		owner()->boardmacaddress.set(owner()->feslots.at(i),owner()->DAQParams.FEParams_v.at(i).FE8MACAdd);
        	}
        	for(int i=0;i<owner()->nledboard();i++){
        		owner()->boardmacaddress.set(owner()->ledslots.at(i),owner()->DAQParams.ledMACAdd);
        	}
        	owner()->isconfigured=true;
		} else if (*command == START_COMMAND) {
			daq->DoStart(owner()->DAQParams);
			cout << "DoStart ok" << endl;
			daq->preQtAcqLoop();
			m_TimeCounter.clear();
			owner()->nbevent=0;
			owner()->started=true;
		}
		else if (*command == READ_COMMAND) {
			daq->QtAcqLoop();
			owner()->nbevent=daq->getNbEvent();
		}
		else if (*command == READINFO_COMMAND) {
			unsigned int accepted_cnt,rejected_cnt;
			daq->QtAcqLoopCounter(accepted_cnt,rejected_cnt,false);
			cout << dec << "Trig accepted " << accepted_cnt << " rejected " << rejected_cnt << endl;
		}else if (*command == STOP_COMMAND) {
			owner()->startActivated =false;
			owner()->started=false;
			cout << "stop " << endl;
			daq->DoStop();
			cout << "save " << endl;
			daq->DoSave();
			owner()->m_StopActivated = false;
		}

        delete command;
	}
}
/*
 * init
 */
void RealStereoElecDriver::init() {


}

/*
 * clear
 */
void RealStereoElecDriver::clear() {

}

/*
 * writeParam
 */
void RealStereoElecDriver::writeParam() {

}


/*
 * readInfos
 */
void RealStereoElecDriver::readInfos() {
	if(owner()->started) m_commandQueue->put(new int32(READINFO_COMMAND));
}
/*
 * start
 */
void RealStereoElecDriver::start() {
	if(owner()->isconfigured()){
		m_commandQueue->put(new int32(START_COMMAND));
		owner()->startActivated=true;
	}else{
		cout << "You have to configure the  MTCA crate (Settings/DAQ_config)" << endl;
	}
}

/*
 * stop
 */
void RealStereoElecDriver::stop() {
	if(owner()->started){
		if(!owner()->m_StopActivated){
			m_commandQueue->put(new int32(STOP_COMMAND));
		}
		owner()->m_StopActivated=true;
	}
}

/*
 * read
 */
void RealStereoElecDriver::read() {
	if(owner()->started) m_commandQueue->put(new int32(READ_COMMAND));
}

/*
 * synchroniseRead
 */
void RealStereoElecDriver::synchroniseRead() {
	if(owner()->started) read();
	owner()->m_SynchroniseReadTerminated = true;
}

/*
 * readStatus
 */
void RealStereoElecDriver::readStatus() {
	//cout << "readStatus() : startActivated = " << owner()->startActivated() << " started " << owner()->started << endl;
	if (owner()->started == true) {
		bool finished = false;
		int32 mode = owner()->mode();

		float64 actual = m_TimeCounter.getTime();
		owner()->time.update(actual);
		if (mode == acquisition::GateCommon::TIME_MODE) {
			float64 target = owner()->time.setpoint();
		   // cout << "Actual time " << actual << " target time " << target << endl;
			if (actual >= target) {
				finished = true;
				actual = target;
			}

		} else if (mode == acquisition::GateCommon::COUNTER1_MODE) {
			int32 target = owner()->counts.setpoint();
			int32 actual = owner()->nbevent;
		//	cout << "Actual nb event " << actual << " target nb event " << target << endl;
			if (actual >= target) {
				finished = true;
			}
		} else if (mode == acquisition::GateCommon::COUNTER2_MODE) {
			int32 target = owner()->counts2.setpoint();
			int32 actual = owner()->nbevent;
		//	cout << "Actual nb event " << actual << " target nb event " << target << endl;
			if (actual >= target) {
				finished = true;
			}
		}

		if (finished) {
			//m_Status = COUNT_END;
			//owner()->status.update(m_Status);
			//owner()->computeGateStatus(m_Status);
			//owner()->computeAcquisitionStatus(0);
			owner()->computeDetectorStatus(0);
			//owner()->m_StopActivated = true;
			//owner()->startActivated = false;
			owner()->commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
			stop();
		} else {
			//owner()->status.update(m_Status);
			//owner()->computeGateStatus(m_Status);
			//owner()->computeAcquisitionStatus(0);
			owner()->computeDetectorStatus(0);
			owner()->calculateProgression();
		}
	} else if(owner()->startActivated()){  // If a run is currently starting, nothing is done
		owner()->computeDetectorStatus(0);
		owner()->calculateProgression();
	} else{
		if (owner()->m_SynchroniseReadActivated == true) {
			if (owner()->m_SynchroniseReadTerminated == true) {
				owner()->m_SynchroniseReadActivated = false;
				owner()->m_SynchroniseReadTerminated = false;
				owner()->commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
			}
		} else {
			owner()->commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
		}
		//m_Status = COUNT_END;
		//owner()->status.update(m_Status);
		//owner()->computeGateStatus(m_Status);
		//owner()->computeAcquisitionStatus(0);
		owner()->computeDetectorStatus(0);
	}
	//cout << "end readstatus() " << endl;
}

void RealStereoElecDriver::config() {
	m_commandQueue->put(new int32(CONFIG_COMMAND));
}


}
